/*
 * 本範程式為Arduino結合2.13吋黑白電子紙的範例程式，由傑森創工提供
 * 本程式由GxEPD2範例程式簡化改寫，若需要進一步學習，建議參考GxEPD2_Example範例
 * 
 * 1，請先安裝Adafruit GFX Library函式庫，可由Arduino IDE裡的「程式管理員」進行安裝
 * 2，請先安裝GxEPD2函式庫，可由Arduino IDE裡的「程式管理員」進行安裝
 * 3，本範例已準備好多個不同解析度的影像檔案，使用者可以之後自行更換
 * 
 * 粉絲團：https://www.facebook.com/jasonshow
 * 傑森創工購物網：https://www.jmaker.com.tw/
 * 傑森創工部落格：https://blog.jmaker.com.tw/
 */

// 接線指示： AVR類型開發板，如：Arduino UNO、NANO 等
// BUSY -> 7, RST -> 9, DC -> 8, CS-> 10, CLK -> 13, DIN -> 11

// 接線指示：Arduino MEGA
// BUSY -> 7, RST -> 9, DC -> 8, CS-> 53, CLK -> 52, DIN -> 51

#define ENABLE_GxEPD2_GFX 0

#include <GxEPD2_BW.h>
#include <GxEPD2_3C.h>
#include <Fonts/FreeSerif9pt7b.h>

//如果使用Arduino Uno或Nano開發板，請在以下選擇電子紙的型號
//GxEPD2_213_B72，2.13吋黑白電子紙
#define MAX_DISPLAY_BUFFER_SIZE 800 // 
#define MAX_HEIGHT(EPD) (EPD::HEIGHT <= MAX_DISPLAY_BUFFER_SIZE / (EPD::WIDTH / 8) ? EPD::HEIGHT : MAX_DISPLAY_BUFFER_SIZE / (EPD::WIDTH / 8))
GxEPD2_BW<GxEPD2_213_B72, MAX_HEIGHT(GxEPD2_213_B72)> display(GxEPD2_213_B72(/*CS=10*/ SS, /*DC=*/ 8, /*RST=*/ 9, /*BUSY=*/ 7)); // GDEH0213B72

#include "GxEPD2_boards_added.h"

//載入圖形檔案
#include "jm01.h" // 122x250,2.13吋電子紙尺寸滿版畫面

void setup()
{
  Serial.begin(115200);
  Serial.println();
  Serial.println("setup");
  delay(100);
  display.init(115200);
  helloWorld();
  delay(1000);
  drawBitmaps();
  partialText();
  Serial.println("setup done");
}

void loop()
{
}

//顯示文字前先清除畫面成白色
void helloWorld()
{
  //Serial.println("helloWorld");
  //是否旋轉畫面，若是旋轉，在顯示圖片前要再旋轉回來，display.setRotation(0);
  //display.setRotation(1); 
  display.setFont(&FreeSerif9pt7b); //設定字型
  display.setTextColor(GxEPD_BLACK); //設定文字顏色為黑色
  //以下是計算文字的範圍，並將整段文字放在畫面中央
  int16_t tbx, tby; uint16_t tbw, tbh;
  display.getTextBounds("Hello World!", 0, 0, &tbx, &tby, &tbw, &tbh);
  uint16_t x = ((display.width() - tbw) / 2) - tbx;
  uint16_t y = ((display.height() - tbh) / 2) - tby;
  display.setFullWindow();
  display.firstPage();
  do
  {
    display.fillScreen(GxEPD_WHITE);
    display.setCursor(x, y); //設定文字開始的座標
    display.print("Hello World!"); //顯示文字
  }
  while (display.nextPage());
  //Serial.println("helloWorld done");
}

//顯示局部文字，不清除畫面
void partialText()
{
  display.setFont(&FreeSerif9pt7b);
  display.setTextColor(display.epd2.hasColor ? GxEPD_RED : GxEPD_BLACK);
  display.setPartialWindow(0, 214, 250, 50); //設定局部文字的範圍
  display.firstPage();
  do
  {
    display.fillScreen(GxEPD_WHITE);
    display.setCursor(5, 244);
    display.print("jmaker.com.tw");
  }
  while (display.nextPage());
  //Serial.println("helloEpaper done");
}

//載入外部圖形檔
void drawBitmaps()
{
  display.setFullWindow();
  if ((display.epd2.panel == GxEPD2::GDE0213B1) || (display.epd2.panel == GxEPD2::GDEH0213B72))
  {
      display.firstPage();
      do
      {
        display.fillScreen(GxEPD_WHITE);
        display.drawBitmap(0, 0, gImage_jm01, display.epd2.WIDTH, display.epd2.HEIGHT, GxEPD_BLACK);
      }
      while (display.nextPage());
      delay(2000);
  }
}
